/**
 * @fileOverview loader object.
 * @version 1.0.0
 * @author izrie
 */

/**
 * loader module.
 * @see <a href="http://play.daumcorp.com/pages/viewpage.action?pageId=105941118" target="_blank">http://play.daumcorp.com/pages/viewpage.action?pageId=105941118</a>
 * @name daum.g.loader
 * @constructor
 *
 * @property {Object} modules 모듈 정보. key => value
 */
new (function() {
     // add private members to _daumGlue
     _daumGlue._commandNextId = 1;
     _daumGlue._commands = {};
     _daumGlue._addCommand = function (module, action, options, params, onSuccess, onError) {
         this._commands[this._commandNextId] = {
             commandId : this._commandNextId,
             module : module,
             action : action,
             options : options,
             params : params,
             success : onSuccess,
             error : onError,
             state : 0
         };
         return this._commandNextId++;
     };
     _daumGlue._execCommandCallback = function (commandId, value, success) {
         var command = this._commands[commandId];
         this._removeCommand(commandId);
         
         if (command) {
             var callback = null;
             if (success === true || success === undefined) {
                 callback = command["success"];
             }
             else if (success === false) {
                 callback = command["error"];
             }
             if (typeof(callback) === "function") {
                 callback(value);
             }
         }
     };
     _daumGlue._removeCommand = function (commandId) {
         delete this._commands[commandId];
     };
     _daumGlue._commandsJSON = function () {
         var keys = [], allCommands = [];
         for (var key in _daumGlue._commands) { keys.push(+key); }
         keys = keys.sort();
         for (var i in keys) {
             allCommands.push(_daumGlue._commands[keys[i]]);
         }
         return JSON.stringify(allCommands);
     };
     _daumGlue._flush = function () {
     	 daumGlueNative.flush(_daumGlue._commandsJSON());
     }
     
     this.modules = {
         "daum.g.code" : {version : "1.0.0"},
         "daum.g.editor" : {version : "1.0.0"},
         "daum.g.geolocation" : {version : "1.0.0"},
         "daum.g.musicSearch" : {version : "1.0.0"},
         "daum.g.objectSearch" : {version : "1.0.0"},
         "daum.g.voiceReco" : {version : "1.0.0"}
     };
     
     /**
      * 모듈을 로드한다.
      * @param {String} moduleName 모듈 이름
      * @param {Function} onSuccess success callback
      * @param {Function} onError error callback
      * @memberOf daum.g.loader#
      */
     this.load = function (moduleName, onSuccess, onError) {
         _daumGlue._addCommand("daum.g.loader", "loadModule", null, {"moduleName" : moduleName}, onSuccess, onError);
         _daumGlue._flush();
     };
})();
